'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var util = require('util');
var stream = require('stream');
var cloneable = _interopDefault(require('cloneable-readable'));
var events = require('events');
var mergeStream = _interopDefault(require('merge-stream'));
var splitStream = _interopDefault(require('split'));
var child_process = require('child_process');
var path = require('path');
var npmRunPath = _interopDefault(require('npm-run-path'));
var getPathKey = _interopDefault(require('path-key'));
var which = _interopDefault(require('which'));
var serializeJavascript = _interopDefault(require('serialize-javascript'));
var t = require('ts-interface-checker');
var net = require('net');

// A type of promise-like that resolves synchronously and supports only one observer
const _Pact = /*#__PURE__*/(function() {
	function _Pact() {}
	_Pact.prototype.then = function(onFulfilled, onRejected) {
		const result = new _Pact();
		const state = this.s;
		if (state) {
			const callback = state & 1 ? onFulfilled : onRejected;
			if (callback) {
				try {
					_settle(result, 1, callback(this.v));
				} catch (e) {
					_settle(result, 2, e);
				}
				return result;
			} else {
				return this;
			}
		}
		this.o = function(_this) {
			try {
				const value = _this.v;
				if (_this.s & 1) {
					_settle(result, 1, onFulfilled ? onFulfilled(value) : value);
				} else if (onRejected) {
					_settle(result, 1, onRejected(value));
				} else {
					_settle(result, 2, value);
				}
			} catch (e) {
				_settle(result, 2, e);
			}
		};
		return result;
	};
	return _Pact;
})();

// Settles a pact synchronously
function _settle(pact, state, value) {
	if (!pact.s) {
		if (value instanceof _Pact) {
			if (value.s) {
				if (state & 1) {
					state = value.s;
				}
				value = value.v;
			} else {
				value.o = _settle.bind(null, pact, state);
				return;
			}
		}
		if (value && value.then) {
			value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
			return;
		}
		pact.s = state;
		pact.v = value;
		const observer = pact.o;
		if (observer) {
			observer(pact);
		}
	}
}

function _isSettledPact(thenable) {
	return thenable instanceof _Pact && thenable.s & 1;
}

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously implement a generic for loop
function _for(test, update, body) {
	var stage;
	for (;;) {
		var shouldContinue = test();
		if (_isSettledPact(shouldContinue)) {
			shouldContinue = shouldContinue.v;
		}
		if (!shouldContinue) {
			return result;
		}
		if (shouldContinue.then) {
			stage = 0;
			break;
		}
		var result = body();
		if (result && result.then) {
			if (_isSettledPact(result)) {
				result = result.s;
			} else {
				stage = 1;
				break;
			}
		}
		if (update) {
			var updateValue = update();
			if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
				stage = 2;
				break;
			}
		}
	}
	var pact = new _Pact();
	var reject = _settle.bind(null, pact, 2);
	(stage === 0 ? shouldContinue.then(_resumeAfterTest) : stage === 1 ? result.then(_resumeAfterBody) : updateValue.then(_resumeAfterUpdate)).then(void 0, reject);
	return pact;
	function _resumeAfterBody(value) {
		result = value;
		do {
			if (update) {
				updateValue = update();
				if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
					updateValue.then(_resumeAfterUpdate).then(void 0, reject);
					return;
				}
			}
			shouldContinue = test();
			if (!shouldContinue || (_isSettledPact(shouldContinue) && !shouldContinue.v)) {
				_settle(pact, 1, result);
				return;
			}
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
				return;
			}
			result = body();
			if (_isSettledPact(result)) {
				result = result.v;
			}
		} while (!result || !result.then);
		result.then(_resumeAfterBody).then(void 0, reject);
	}
	function _resumeAfterTest(shouldContinue) {
		if (shouldContinue) {
			result = body();
			if (result && result.then) {
				result.then(_resumeAfterBody).then(void 0, reject);
			} else {
				_resumeAfterBody(result);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
	function _resumeAfterUpdate() {
		if (shouldContinue = test()) {
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
			} else {
				_resumeAfterTest(shouldContinue);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
}

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

const isWindows = process.platform === 'win32' || process.env.OSTYPE === 'cygwin' || process.env.OSTYPE === 'msys';
function spawnProcess(config) {
  const cwd = path.resolve(config.cwd);
  let [binary, ...args] = config.command;
  let env = { ...config.env
  }; // Use uppercase PATH key regardless of OS or original key

  env.PATH = filterBlankParts(npmRunPath({
    cwd,
    path: config.env[getPathKey({
      env: config.env
    })] || ''
  }));

  if (isWindows) {
    env.PATHEXT = config.env.PATHEXT || '.COM;.EXE;.BAT;.CMD;.VBS;.VBE;.JS;.JSE;.WSF;.WSH';
    /*
      Work around issue (same issue):
        - https://github.com/nodejs/node-v0.x-archive/issues/2318
        - https://github.com/nodejs/node/issues/6671
           Without resorting to wrapping command in shell (like npm package `cross-spawn` does)
      because [spawning a shell is expensive](https://github.com/nodejs/node/issues/6671#issuecomment-219210529)
      and more importantly, that approach introduces *more* window-linux disparities.
           Instead just replace `binary` with a fully-qualified version.
     */

    binary = path.normalize(myWhich(cwd, binary, env.PATH, env.PATHEXT) || binary);
  }

  return child_process.spawn(binary, args, {
    cwd,
    env
  });
}

function filterBlankParts(string) {
  const colon = isWindows ? ';' : ':';
  return string.split(colon).filter(Boolean).join(colon);
} // Version of `which.sync` that adds a `cwd` parameter & doesn't throw


function myWhich(cwd, binary, path, pathExt) {
  const originalCwd = process.cwd();

  if (cwd !== originalCwd) {
    process.chdir(cwd);
  }

  const result = which.sync(binary, {
    nothrow: true,
    path,
    pathExt
  });

  if (cwd !== originalCwd) {
    process.chdir(originalCwd);
  }

  return result;
}

function mapStreamLines(callback) {
  return new stream.Transform({
    objectMode: true,

    transform(line, _, cb) {
      this.push(callback(line));
      cb();
    }

  });
}
function tapStreamLines(callback) {
  return new stream.Transform({
    objectMode: true,

    transform(line, _, cb) {
      callback(line);
      this.push(line);
      cb();
    }

  });
}
function filterBlankLastLine(blankLine) {
  let bufferedBlankChunk = false;
  return new stream.Transform({
    objectMode: true,

    transform(line, _, callback) {
      if (bufferedBlankChunk) {
        this.push(blankLine);
        bufferedBlankChunk = false;
      }

      if (line === blankLine) {
        bufferedBlankChunk = true;
      } else {
        this.push(line);
      }

      callback();
    }

  });
}

const delay = /*#__PURE__*/util.promisify(setTimeout);
class ServiceProcess {
  constructor(serviceId, serviceConfig, logger, onCrash) {
    this.logTail = [];
    this.didError = false;
    this.didEnd = false;
    this.wasEndCalled = false;
    this.serviceId = serviceId;
    this.serviceConfig = serviceConfig;
    this.logger = logger;
    this.process = spawnProcess(this.serviceConfig);
    this.started = Promise.race([events.once(this.process, 'error'), delay(100)]).then(result => {
      if (result && result[0]) {
        this.didError = true;
        throw result[0];
      }
    });
    this.output = getProcessOutput(this.process);

    if (this.serviceConfig.logTailLength > 0) {
      this.output = this.output.pipe(tapStreamLines(line => {
        this.logTail.push(line);

        if (this.logTail.length > this.serviceConfig.logTailLength) {
          this.logTail.shift();
        }
      }));
    }

    this.ended = Promise.all([this.started.catch(() => {}), events.once(this.output, 'end').then(() => {
      this.didEnd = true;
    })]).then(() => {
      if (!this.didError && !this.wasEndCalled) {
        onCrash();
      }
    });
  }

  isRunning() {
    return !this.didError && !this.didEnd;
  }

  end(windowsCtrlCShutdown) {
    if (!this.wasEndCalled) {
      this.wasEndCalled = true;

      if (this.isRunning()) {
        if (windowsCtrlCShutdown) {
          // Don't call this.process.kill(); ctrl+c was already sent to all services
          this.forceKillAfterTimeout();
        } else if (process.platform === 'win32') {
          this.process.kill(); // Don't call this.forceKillAfterTimeout(); On Windows we don't have SIGINT vs SIGKILL
        } else {
          this.process.kill('SIGINT');
          this.forceKillAfterTimeout();
        }
      }
    }

    return this.ended;
  }

  forceKillAfterTimeout() {
    if (this.serviceConfig.forceKillTimeout === Infinity) {
      return;
    }

    setTimeout(() => {
      if (this.isRunning()) {
        this.logger.info(`Force killing service '${this.serviceId}'`);
        this.process.kill('SIGKILL');
      }
    }, this.serviceConfig.forceKillTimeout);
  }

}

function getProcessOutput(proc) {
  return mergeStream([proc.stdout, proc.stderr].map(stream => stream.setEncoding('utf8')).map(stream$1 => stream.pipeline(stream$1, splitStream(line => `${line}\n`), filterBlankLastLine('\n'), () => {})));
}

const genericMessage = 'This is a bug in composite-service. Please file an issue in https://github.com/zenflow/composite-service/issues';
class InternalError extends Error {
  constructor(message) {
    super(`${message}. ${genericMessage}`);
    Object.setPrototypeOf(this, InternalError.prototype);
  }

}
InternalError.prototype.name = InternalError.name;

const delay$1 = /*#__PURE__*/util.promisify(setTimeout);
class Service {
  constructor(id, config, logger, handleError) {
    this.output = cloneable(new stream.PassThrough({
      objectMode: true
    }));
    this.outputClone = this.output.clone();
    this.crashes = [];
    this.id = id;
    this.config = config;
    this.logger = logger;
    this.handleError = handleError;
  }

  start() {
    if (this.stopResult) {
      this.logger.error(new InternalError('Cannot start after stopping').stack);
      return this.startResult;
    }

    if (!this.startResult) {
      this.logger.info(`Starting service '${this.id}'...`);
      this.defineReady();
      this.startResult = this.startProcess().then(() => this.ready).then(() => {
        this.logger.info(`Started service '${this.id}'`);
      });
    }

    return this.startResult;
  }

  defineReady() {
    const ctx = {
      output: this.outputClone
    };
    this.ready = promiseTry(() => this.config.ready(ctx)).catch(error => this.handleError(`Error from ready function: ${maybeErrorText(error)}`)).then(() => {
      this.outputClone.destroy();
    });
  }

  startProcess() {
    try {
      const _this = this;

      const proc = new ServiceProcess(_this.id, _this.config, _this.logger, () => {
        proc.output.unpipe();

        if (!_this.stopResult) {
          _this.handleCrash(proc);
        }
      });
      _this.process = proc;
      proc.output.pipe(_this.output, {
        end: false
      });

      const _temp = _catch(function () {
        return Promise.resolve(_this.process.started).then(function () {});
      }, function (error) {
        return Promise.resolve(_this.handleError(`Error starting process: ${error}`)).then(function () {});
      });

      return Promise.resolve(_temp && _temp.then ? _temp.then(function () {}) : void 0);
    } catch (e) {
      return Promise.reject(e);
    }
  }

  handleCrash(proc) {
    try {
      const _this2 = this;

      _this2.logger.info(`Service '${_this2.id}' crashed`);

      const delayPromise = delay$1(_this2.config.minimumRestartDelay);
      const crash = {
        date: new Date(),
        logTail: proc.logTail
      };

      _this2.crashes.push(crash);

      return Promise.resolve(isResolved(_this2.ready)).then(function (isServiceReady) {
        function _temp3() {
          if (_this2.stopResult) {
            return;
          }

          return Promise.resolve(delayPromise).then(function () {
            if (_this2.stopResult) {
              return;
            }

            _this2.logger.info(`Restarting service '${_this2.id}'`);

            return Promise.resolve(_this2.startProcess()).then(function () {});
          });
        }

        const ctx = {
          isServiceReady,
          crash,
          crashes: _this2.crashes
        };

        const _temp2 = _catch(function () {
          return Promise.resolve(_this2.config.onCrash(ctx)).then(function () {});
        }, function (error) {
          return Promise.resolve(_this2.handleError(`Error from onCrash function: ${maybeErrorText(error)}`)).then(function () {});
        });

        return _temp2 && _temp2.then ? _temp2.then(_temp3) : _temp3(_temp2);
      });
    } catch (e) {
      return Promise.reject(e);
    }
  }

  stop(windowsCtrlCShutdown) {
    if (!this.stopResult) {
      if (!this.process || !this.process.isRunning()) {
        this.stopResult = Promise.resolve();
      } else {
        this.logger.info(`Stopping service '${this.id}'...`);
        this.stopResult = this.process.end(windowsCtrlCShutdown).then(() => {
          this.logger.info(`Stopped service '${this.id}'`);
        });
      }
    }

    return this.stopResult;
  }

}
function maybeErrorText(maybeError) {
  return maybeError instanceof Error && maybeError.stack || String(maybeError);
}

function promiseTry(fn) {
  try {
    return Promise.resolve(fn());
  } catch (error) {
    return Promise.reject(error);
  }
}

function isResolved(promise) {
  return Promise.race([promise.then(() => true, () => false), Promise.resolve().then(() => false)]);
}

function once(fn) {
  var result;
  return function () {
    return result || (result = fn());
  };
}

var typeSuite0 = /*#__PURE__*/once(function () {
  return {
    CompositeServiceConfig: t.iface([], {
      "logLevel": t.opt(t.union(t.lit('debug'), t.lit('info'), t.lit('error'))),
      "gracefulShutdown": t.opt("boolean"),
      "windowsCtrlCShutdown": t.opt("boolean"),
      "serviceDefaults": t.opt("ServiceConfig"),
      "services": t.iface([], {})
    })
  };
});
var typeSuite1 = /*#__PURE__*/once(function () {
  return {
    ServiceConfig: t.iface([], {
      "dependencies": t.opt(t.array("string")),
      "cwd": t.opt("string"),
      "command": t.opt(t.union("string", t.array("string"))),
      "env": t.opt(t.iface([], {})),
      "ready": t.opt(t.func("any", t.param("ctx", "ReadyContext"))),
      "forceKillTimeout": t.opt("number"),
      "onCrash": t.opt(t.func("any", t.param("ctx", "OnCrashContext"))),
      "logTailLength": t.opt("number"),
      "minimumRestartDelay": t.opt("number")
    })
  };
});
function validateAndNormalizeConfig(config) {
  validateType('CompositeServiceConfig', 'config', config);
  const {
    logLevel = 'info'
  } = config;
  const windowsCtrlCShutdown = process.platform === 'win32' && Boolean(config.windowsCtrlCShutdown);
  const gracefulShutdown = !windowsCtrlCShutdown && Boolean(config.gracefulShutdown);
  const {
    serviceDefaults = {}
  } = config;
  doExtraServiceConfigChecks('config.serviceDefaults', serviceDefaults);
  const truthyServiceEntries = Object.entries(config.services).filter(([, value]) => value);

  if (truthyServiceEntries.length === 0) {
    throw new ConfigValidationError('`config.services` has no entries');
  }

  const services = {};

  for (const [id, config] of truthyServiceEntries) {
    services[id] = processServiceConfig(id, config, serviceDefaults);
  }

  validateDependencyTree(services);
  return {
    logLevel,
    gracefulShutdown,
    windowsCtrlCShutdown,
    services
  };
}

function doExtraServiceConfigChecks(path, config) {
  if (typeof config.command !== 'undefined' && (Array.isArray(config.command) ? !config.command.length || !config.command[0].trim() : !config.command.trim())) {
    throw new ConfigValidationError(`\`${path}.command\` has no binary part`);
  }
}

function processServiceConfig(id, config, defaults) {
  const path = `config.services.${id}`;
  validateType('ServiceConfig', path, config);
  doExtraServiceConfigChecks(path, config);
  const merged = {
    dependencies: [],
    cwd: '.',
    // no default command
    env: {},
    ready: () => Promise.resolve(),
    forceKillTimeout: 5000,
    onCrash: () => {},
    logTailLength: 0,
    minimumRestartDelay: 1000,
    ...removeUndefinedProperties(defaults),
    ...removeUndefinedProperties(config)
  };

  if (merged.command === undefined) {
    throw new ConfigValidationError(`\`${path}.command\` is not defined`);
  }

  return { ...merged,
    command: normalizeCommand(merged.command),
    env: normalizeEnv(merged.env)
  };
}

function removeUndefinedProperties(object) {
  const result = { ...object
  };

  for (const [key, value] of Object.entries(result)) {
    if (value === undefined) {
      delete result[key];
    }
  }

  return result;
}

function normalizeCommand(command) {
  return Array.isArray(command) ? command : command.split(/\s+/).filter(Boolean);
}

function normalizeEnv(env) {
  return Object.fromEntries(Object.entries(env).filter(([, value]) => value !== undefined).map(([key, value]) => [key, String(value)]));
}

function validateDependencyTree(services) {
  const serviceIds = Object.keys(services);

  for (const [serviceId, {
    dependencies
  }] of Object.entries(services)) {
    for (const dependency of dependencies) {
      if (!serviceIds.includes(dependency)) {
        throw new ConfigValidationError(`Service "${serviceId}" has dependency on unknown service "${dependency}"`);
      }
    }
  }

  for (const serviceId of serviceIds) {
    validateNoCyclicDeps(serviceId, []);
  }

  function validateNoCyclicDeps(serviceId, path) {
    const isLooped = path.includes(serviceId);

    if (isLooped) {
      throw new ConfigValidationError(`Service "${serviceId}" has cyclic dependency ${path.slice(path.indexOf(serviceId)).concat(serviceId).join(' -> ')}`);
    }

    for (const dep of services[serviceId].dependencies) {
      validateNoCyclicDeps(dep, [...path, serviceId]);
    }

    return null;
  }
}

class ConfigValidationError extends Error {
  constructor(message) {
    super(message);
    Object.setPrototypeOf(this, ConfigValidationError.prototype);
  }

}
ConfigValidationError.prototype.name = ConfigValidationError.name;
const checkers = /*#__PURE__*/t.createCheckers({ ... /*#__PURE__*/typeSuite0(),
  ... /*#__PURE__*/typeSuite1()
});

function validateType(typeName, reportedPath, value) {
  const checker = checkers[typeName];
  checker.setReportedPath(reportedPath);
  const error = checker.validate(value);

  if (error) {
    throw new ConfigValidationError(getErrorMessage(error));
  }
}

function getErrorMessage(error) {
  return getErrorMessageLines(error).join('\n');
}

function getErrorMessageLines(error) {
  let result = [`\`${error.path}\` ${error.message}`];

  if (error.nested) {
    for (const nested of error.nested) {
      result = result.concat(getErrorMessageLines(nested).map(s => `    ${s}`));
    }
  }

  return result;
}

const orderedLogLevels = ['error', 'info', 'debug'];
class Logger {
  constructor(level) {
    this.output = new stream.PassThrough({
      objectMode: true
    });
    this.level = level;
    this.error = this.log.bind(this, 'error');
    this.info = this.log.bind(this, 'info');
    this.debug = this.log.bind(this, 'debug');
  }

  log(level, text) {
    if (this.shouldLog(level)) {
      for (const line of text.split('\n')) {
        this.output.write(`${level}: ${line}\n`);
      }
    }
  }

  shouldLog(level) {
    return orderedLogLevels.indexOf(level) <= orderedLogLevels.indexOf(this.level);
  }

}

class CompositeService {
  constructor(config) {
    this.stopping = false;
    this.config = validateAndNormalizeConfig(config);

    if (process.platform === 'win32' && this.config.windowsCtrlCShutdown) {
      require('generate-ctrl-c-event'); // make sure this module loads before we even start

    }

    const outputStream = mergeStream();
    outputStream.pipe(process.stdout);
    this.logger = new Logger(this.config.logLevel);
    outputStream.add(this.logger.output);
    this.logger.debug('Config: ' + serializeJavascript(config, {
      space: 2,
      unsafe: true
    }));
    process.on('SIGINT', () => this.die(130, "Received 'SIGINT' signal"));
    process.on('SIGTERM', () => this.die(143, "Received 'SIGTERM' signal"));

    if (process.stdin.isTTY) {
      process.stdin.setRawMode(true);
      process.stdin.on('data', buffer => {
        if (buffer.toString('utf8') === '\u0003') {
          this.die(130, 'Received ctrl+c');
        }
      });
    }

    this.services = Object.entries(this.config.services).map(([id, config]) => new Service(id, config, this.logger, message => this.handleError(`Error in '${id}': ${message}`)));
    this.serviceMap = new Map(this.services.map(service => [service.id, service]));
    const maxLabelLength = Math.max(...Object.keys(this.config.services).map(({
      length
    }) => length));
    outputStream.add(this.services.map(({
      output,
      id
    }) => output.pipe(mapStreamLines(line => `${`${rightPad(id, maxLabelLength)} | `}${line}`))));
    this.logger.info('Starting composite service...');
    Promise.all(this.services.map(service => this.startService(service))).then(() => this.logger.info('Started composite service'));
  }

  startService(service) {
    try {
      const _this = this;

      const dependencies = service.config.dependencies.map(id => _this.serviceMap.get(id));
      return Promise.resolve(Promise.all(dependencies.map(service => _this.startService(service)))).then(function () {
        function _temp3() {
          return Promise.resolve(service.start()).then(function () {
            const _temp = function () {
              if (_this.stopping) {
                return Promise.resolve(never()).then(function () {});
              }
            }();

            if (_temp && _temp.then) return _temp.then(function () {});
          });
        }

        const _temp2 = function () {
          if (_this.stopping) {
            return Promise.resolve(never()).then(function () {});
          }
        }();

        return _temp2 && _temp2.then ? _temp2.then(_temp3) : _temp3(_temp2);
      });
    } catch (e) {
      return Promise.reject(e);
    }
  }

  handleError(message) {
    return this.die(1, message);
  }

  die(exitCode, message) {
    try {
      const _this2 = this;

      const _temp4 = function () {
        if (!_this2.stopping) {
          _this2.stopping = true;
          const isSignalExit = exitCode > 128; // we have either a signal exit or an error exit

          _this2.logger[isSignalExit ? 'info' : 'error'](message);

          _this2.logger.info('Stopping composite service...');

          if (_this2.config.windowsCtrlCShutdown) {
            require('generate-ctrl-c-event').generateCtrlCAsync().catch(error => _this2.logger.error(String(error)));
          }

          return Promise.resolve(Promise.all(_this2.services.map(service => _this2.stopService(service)))).then(function () {
            _this2.logger.info('Stopped composite service');

            return Promise.resolve(Promise.resolve()).then(function () {
              // Wait one micro tick for output to flush
              process.exit(exitCode);
            });
          });
        }
      }();

      return Promise.resolve(_temp4 && _temp4.then ? _temp4.then(function () {
        // simply return a promise that never resolves, since we can't do anything after exiting anyways
        return never();
      }) : never());
    } catch (e) {
      return Promise.reject(e);
    }
  }

  stopService(service) {
    try {
      const _this3 = this;

      function _temp6() {
        return Promise.resolve(service.stop(_this3.config.windowsCtrlCShutdown)).then(function () {});
      }

      const _temp5 = function () {
        if (_this3.config.gracefulShutdown) {
          const dependents = _this3.services.filter(({
            config
          }) => config.dependencies.includes(service.id));

          return Promise.resolve(Promise.all(dependents.map(service => _this3.stopService(service)))).then(function () {});
        }
      }();

      return Promise.resolve(_temp5 && _temp5.then ? _temp5.then(_temp6) : _temp6(_temp5));
    } catch (e) {
      return Promise.reject(e);
    }
  }

}

function rightPad(string, length) {
  // we assume length >= string.length
  return string + ' '.repeat(length - string.length);
}

function never() {
  return new Promise(() => {});
}

let started = false;
/**
 * Starts a composite service in the current Node.js process
 *
 * @remarks
 *
 * Each service defined in {@link CompositeServiceConfig.services}
 * is started and managed according to its {@link ServiceConfig}.
 *
 * The stdout & stderr from each service is piped to stdout,
 * each line prepended with the service ID.
 *
 * Each service is told to shut down with `SIGINT`
 * when the composite service is itself told to shut down
 * (with `ctrl+c`, `SIGINT`, or `SIGTERM`),
 * or when a fatal error occurs.
 *
 * All possible fatal errors:
 * (1) A composite service was already started in this process
 * (2) Error validating configuration
 * (3) Error spawning process, e.g. `EPERM`, `ENOENT`, etc.
 * (4) Error from user-provided configuration function,
 * {@link ServiceConfig.ready} or {@link ServiceConfig.onCrash}
 *
 * @param config - Configuration for the composite service
 *
 * @public
 */

function startCompositeService(config) {
  if (started) {
    throw new Error('Already started a composite service in this process');
  }

  new CompositeService(config);
  started = true;
}

/**
 * Waits until `output` emits a `line` for which `test(line)` returns `true`
 *
 * @param output - To scan
 * @param test - To test each `line` from `output`
 *
 * @example
 *
 * ```js
 * const { onceOutputLine } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node service.js',
 *   ready: ctx => onceOutputLine(ctx.output, line => line.match(/^Ready/)),
 * }
 * ```
 *
 * @public
 */
function onceOutputLine(output, test) {
  return new Promise(resolve => {
    const handler = line => {
      if (test(line)) {
        output.off('data', handler);
        resolve();
      }
    };

    output.on('data', handler);
  });
}
/**
 * Waits until `output` emits a line equal to (`===`) `value`
 *
 * @param output - To scan
 * @param value - To look for in `output`
 *
 * @example
 *
 * ```js
 * const { onceOutputLine } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node service.js',
 *   ready: ctx => onceOutputLineIs(ctx.output, 'Ready\n'),
 * }
 * ```
 *
 * @public
 */

function onceOutputLineIs(output, value) {
  return onceOutputLine(output, line => line === value);
}
/**
 * Waits until `output` emits a line that `.includes(value)`
 *
 * @param output - To scan
 * @param value - To look for within each line from `output`
 *
 * @example
 *
 * ```js
 * const { onceOutputLineIncludes } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node service.js',
 *   ready: ctx => onceOutputLineIncludes(ctx.output, 'Ready'),
 * }
 * ```
 *
 * @public
 */

function onceOutputLineIncludes(output, value) {
  return onceOutputLine(output, line => line.includes(value));
}

/**
 * Waits until the given `port` (on the given `host`) is accepting connections
 *
 * @remarks
 *
 * Works by trying to connect to the given port
 * (as opposed to trying to listen on the given port)
 * every 250 milliseconds.
 *
 * @param port -
 * @param host - Defaults to `'localhost'`
 *
 * @example
 *
 * ```js
 * const { onceTcpPortUsed } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node server.js',
 *   env: { PORT: 3000 },
 *   ready: () => onceTcpPortUsed(3000),
 * }
 * ```
 *
 * @public
 */
const onceTcpPortUsed = function (port, host = 'localhost') {
  try {
    let _exit = false;
    const portNumber = typeof port === 'number' ? port : parseInt(port, 10);

    const _temp2 = _for(function () {
      return !_exit;
    }, void 0, function () {
      return Promise.resolve(isTcpPortUsed(portNumber, host)).then(function (_isTcpPortUsed) {
        const _temp = function () {
          if (_isTcpPortUsed) {
            _exit = true;
          } else {
            return Promise.resolve(delay$2(250)).then(function () {});
          }
        }();

        if (_temp && _temp.then) return _temp.then(function () {});
      });
    });

    return Promise.resolve(_temp2 && _temp2.then ? _temp2.then(function () {}) : void 0);
  } catch (e) {
    return Promise.reject(e);
  }
};
const delay$2 = /*#__PURE__*/util.promisify(setTimeout);

function isTcpPortUsed(port, host) {
  return new Promise((resolve, reject) => {
    const socket = new net.Socket();
    socket.once('connect', () => {
      resolve(true);
      cleanUp();
    });
    socket.once('error', error => {
      if (['ECONNREFUSED', 'ETIMEDOUT'].includes(error.code)) {
        resolve(false);
      } else {
        reject(error);
      }

      cleanUp();
    });

    function cleanUp() {
      socket.removeAllListeners();
      socket.end();
      socket.destroy();
      socket.unref();
    }

    socket.connect(port, host);
  });
}

/**
 * Waits a predetermined length of time
 *
 * @param milliseconds - Length of time to wait
 *
 * @example
 *
 * ```js
 * const { onceTimeout } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node service.js',
 *   ready: () => onceTimeout(1000),
 * }
 * ```
 *
 * @public
 */
const onceTimeout = function (milliseconds) {
  try {
    return Promise.resolve(delay$3(milliseconds)).then(function () {});
  } catch (e) {
    return Promise.reject(e);
  }
};
const delay$3 = /*#__PURE__*/util.promisify(setTimeout);

/**
 * Generates a {@link ServiceConfig} for an HTTP gateway service
 *
 * @param config - Configuration for the HTTP gateway service
 *
 * @remarks
 *
 * The HTTP gateway service will use an array of
 * [http-proxy-middleware](https://github.com/chimurai/http-proxy-middleware)
 * instances to forward incoming HTTP requests to the appropriate composed (or external)
 * service as determined by the {@link HttpGatewayConfig.proxies | proxies} option.
 *
 * The service can safely be used as a dependency of other services,
 * since its configuration includes {@link ServiceConfig.ready}.
 *
 * @public
 */

function configureHttpGateway(config) {
  const {
    dependencies,
    host,
    port,
    proxies
  } = config;
  return {
    dependencies,
    command: ['node', `${__dirname}/http-gateway-server.js`],
    env: {
      HOST: host || '0.0.0.0',
      PORT: String(port),
      PROXIES: serializeJavascript(proxies, {
        unsafe: true
      })
    },
    ready: ctx => onceOutputLineIncludes(ctx.output, 'Listening @ http://')
  };
}

exports.configureHttpGateway = configureHttpGateway;
exports.onceOutputLine = onceOutputLine;
exports.onceOutputLineIncludes = onceOutputLineIncludes;
exports.onceOutputLineIs = onceOutputLineIs;
exports.onceTcpPortUsed = onceTcpPortUsed;
exports.onceTimeout = onceTimeout;
exports.startCompositeService = startCompositeService;
//# sourceMappingURL=index.js.map
