/**
 * Waits until the given `port` (on the given `host`) is accepting connections
 *
 * @remarks
 *
 * Works by trying to connect to the given port
 * (as opposed to trying to listen on the given port)
 * every 250 milliseconds.
 *
 * @param port -
 * @param host - Defaults to `'localhost'`
 *
 * @example
 *
 * ```js
 * const { onceTcpPortUsed } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node server.js',
 *   env: { PORT: 3000 },
 *   ready: () => onceTcpPortUsed(3000),
 * }
 * ```
 *
 * @public
 */
export declare function onceTcpPortUsed(port: number | string, host?: string): Promise<void>;
//# sourceMappingURL=port-used.d.ts.map